// written by @monkeydom at cccamp2011
// modified by nuess0r to drive a connected BLINKENmini
// Information about the BLINKENmini:
//   http://blinkenmini.schuermans.info

#include <sysinit.h>

#include "basic/basic.h"
#include "lcd/lcd.h"
#include "lcd/display.h"
#include "lcd/print.h"
#include "filesystem/ff.h"
#include "filesystem/select.h"

#include "usetable.h"

#define SAVE_KEY_REPEAT delayms(100)

void showHDL(void);
uint8_t showHDLBLM(char *filename, bool loop);
void showAllMovies();
void mainScreen();


void blinkenminiInit();

// Function to write a column to the BLINKENmini
void blinkenminiSetCol(bool column[8]);

// Function to finish a write to the BLINKENmini
void blinkenminiDisplay();



//shows *.lcd
void ram(void)
{
    char filename[13];
    char key;

    // Initialize GPIO Pins etc. for the BLINKENmini
    blinkenminiInit();

    while (1) {
    	SAVE_KEY_REPEAT;
    	mainScreen();
		key = getInputWait();
	
		// Show file
		if (key == BTN_RIGHT) {
			// Select file
			lcdClear();
			delayms(230);
			selectFile(filename,"BLM");
			// Load as animation
			showHDLBLM(filename, true);
		} else if (key == BTN_DOWN) {    
			// Select file
			lcdClear();
			//delayms(230);
			showAllMovies();
		} else if (key == BTN_ENTER) {
			showHDL();
		} else if (key == BTN_LEFT) {
			// Exit
			return;
		}
	}
}

void mainScreen() {
	getInputWaitRelease();
	while (!getInputRaw()) {
		lcdLoadImage("blinken0.lcd"); // 800ms
		lcdRefresh();
		getInputWaitTimeout(800);
		if (getInputRaw()) return;
		lcdLoadImage("blinken1.lcd"); // 200ms
		lcdRefresh();
		getInputWaitTimeout(200);
	}
}

void drawWindow(int x, int y, int on) {
	int originX = x * 5+4;
	int originY = y * 8+3;
	for (int x = originX; x < originX+ 4; x++) {
		for (int y = originY; y < originY+5; y++) {
			lcdSetPixel(x,y,on);
		}
	}
}

void showHDL(void) {
//    char blinkenHDLFrame[8][18] = {
//    	{0,0,0,0,1,1,1,0,0,0,0,1,1,1,0,0,0,0},
//		{0,0,1,1,1,1,1,1,0,0,1,1,1,1,1,1,0,0},
//		{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
//		{0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
//		{0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0},
//		{0,0,0,0,1,1,1,1,1,1,1,1,1,1,0,0,0,0},
//		{0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0},
//		{0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0}
//    };
    char key;
    
	lcdFill(0);
	while (1) {
		for (int x=0; x<18; x++) {
			for (int y=0; y<8; y++) {
				drawWindow(x,y,true);//blinkenHDLFrame[y][x]);
			}
		}
		lcdDisplay();
		
		SAVE_KEY_REPEAT;
		key = getInputWait();
		if (key == BTN_LEFT) return;
		for (int x=0; x<18; x++) {
			for (int y=0; y<8; y++) {
				drawWindow(x,y,false);//1-blinkenHDLFrame[y][x]);
			}
		}
		lcdDisplay();
		
		SAVE_KEY_REPEAT;
		key = getInputWait();
		if (key == BTN_LEFT) return;

	}
}

#define SEARCH_AT 1
#define PARSE_DURATION 2
#define GOTO_NEWLINE 3
#define PARSE_FRAME 4

#define MAX_BUF_SIZE (18+20)*2

void showHDLFrame(bool frame[8][18]) {
  bool column[8];

//	lcdFill(0);
  // Because we shift all the colums through the shift registers in the 
  // BLINKENmini we have to start with the rightmost column.
        //for (int x=0; x<18; x++) {
	for (int x=17; x>=0; x--) {
		for (int y=0; y<8; y++) {
			drawWindow(x,y,frame[y][x]);
			column[y] = frame[y][x];
		}
		blinkenminiSetCol(column);

	}
	lcdDisplay();
	blinkenminiDisplay();
}

#define PERPAGE 10
#define FLEN 13

void showAllMovies() {
	while(1) {
		int skip = 0;
        char files[PERPAGE][FLEN];
        int count = getFiles(files, PERPAGE, skip, "BLM");
		for (int filePos = 0; filePos < count; filePos++) {
			showHDLBLM(files[filePos],false);
			if (getInputRaw() == BTN_LEFT) return;
		}
		skip++;
		if (count == 0) {
			skip = 0;
		}
	}
}

uint8_t showHDLBLM(char *filename, bool loop) {
	FIL file;
	int result;
	UINT readBytes;
	char buffer[MAX_BUF_SIZE];
	UINT buffer_offset = 0;
	uint8_t state = SEARCH_AT;
	int duration = 0;
	bool frame[8][18];
	char key=0;
	int8_t framex = 0;
	int8_t framey = 0;
	
	
	int frameCount = 0;
	int frameCountFile = 0;
	lcdClear();
	lcdFill(0);

	result = f_open(&file, filename, FA_OPEN_EXISTING|FA_READ);
	if (result) return 1;
	
	while (1) {
		result = f_read(&file,buffer,MAX_BUF_SIZE,&readBytes);
//		lcdPrintInt(readBytes);
//		lcdPrintln("");
		if (readBytes == 0) {
			if (loop) {
				f_lseek(&file,0);
				frameCountFile = 0;
				continue;
			} else {
				return 0;
			}
		}
		if (result) return -1;
//		lcdPrintln("Read state...");
		buffer_offset = 0;
		while (buffer_offset < readBytes) {
			switch (state) {
				case SEARCH_AT:
//					lcdPrint(IntToStr(frameCountFile,3,0));
//					lcdPrint(IntToStr(frameCount,2,0));
//					lcdPrint("sa");
					while (buffer_offset < readBytes &&
					       buffer[buffer_offset] != '#' &&
					       buffer[buffer_offset] != '@') buffer_offset++;
					if (buffer[buffer_offset] == '#') {
						state = GOTO_NEWLINE;
					} else if (buffer[buffer_offset] == '@') {
						gpioSetValue (RB_LED1, 1); 
						state = PARSE_DURATION;
						duration = 0;
					}
					buffer_offset++;
					break;
				case GOTO_NEWLINE:
//					lcdPrint(IntToStr(frameCountFile,3,0));
//					lcdPrint(IntToStr(frameCount,2,0));
//					lcdPrint("gn");
					while (buffer_offset < readBytes &&
					       buffer[buffer_offset] != '\n') buffer_offset++;
					if (buffer_offset < readBytes && buffer[buffer_offset] == '\n') {
						state = SEARCH_AT;
						buffer_offset++;
					}
					break;
				case PARSE_DURATION:
////					lcdPrint(IntToStr(frameCountFile,3,0));
////					lcdPrint(IntToStr(frameCount,2,0));
////					lcdPrint("pd");
					while (buffer_offset < readBytes &&
					       buffer[buffer_offset] >= '0' && buffer[buffer_offset] <='9') {
						duration = duration * 10 + (buffer[buffer_offset++] - '0');
					}

					if (buffer_offset != readBytes) {
						state = PARSE_FRAME;
						framex = 0;
						framey = -1;
					}
					break;
				case PARSE_FRAME:
//					lcdPrint(IntToStr(frameCountFile,3,0));
//					lcdPrint("-");
//					lcdPrint(IntToStr(frameCount,2,0));
//					lcdPrint("pf");
					while (buffer_offset < readBytes) {
						char byte = buffer[buffer_offset];
						if (byte == '\n') {
							framey++;
							framex=0;
							if (framey >= 9) {
								frameCount++;
								frameCountFile++;
								// draw it baby
								showHDLFrame(frame);
								key = getInputWaitTimeout(duration > 50 ? duration : 50);
								gpioSetValue (RB_LED1, 0); 
								if (key == BTN_LEFT) return 0;
								else getInputWaitRelease();
								state = SEARCH_AT;
								break;
							}
						} else if (byte == '0' || byte == '1') {
							frame[framey][framex++] = (byte != '0');
						}
						buffer_offset++;
					}
					break;
			}
		}
	}
	
}

/******************************************************************************
 * How to connect your BLINKENmini to the r0ket:
 *  You need some level shifters between the GPIOs of the r0ket an the parallel 
 *  port input. For example 74HCT245 bus transceivers or other 74HCT chips 
 *  (the T is the important thing here!), use the 5V supply from the BLINKENmini
 *  to power the level shifter.
 *
 * Pinning table:
 * 
 *  Function   r0ket pin      BLINKENmini DB25 input pin    GPIO on LPC1343
 *   Data 0     HB0            2                             PIO1_3
 *   Data 1     HB2            3                             PIO1_1
 *   Data 2     HB4            4                             PIO1_4
 *   Data 3     HB5            5                             PIO1_2
 *   Data 4     SS0            6                             PIO2_5
 *   Data 5     SS1            7                             PIO2_4
 *   Data 6     SS2            8                             PIO2_8
 *   Data 7     SS5            9                             PIO2_10
 *   Clock      SS3            1                             PIO3_2
 *   Strobe     SS4            14                            PIO3_1
 *
 ******************************************************************************/

// Pointers to the GPIO data register using the data adress masking feature
//uint16_t* dataOnPio1;
//uint16_t* dataOnPio2;
//uint16_t* strobeOnPio3;
//uint16_t* clockOnPio3;



//All functions to show the frames on a connected BLINKENmini

void blinkenminiInit()
{
  /* dataOnPio1 = (uint16_t*)(GPIO_GPIO1_BASE | GPIO_IO_P1  */
  /* 				   | GPIO_IO_P2  */
  /* 				   | GPIO_IO_P3  */
  /* 				   | GPIO_IO_P4); */
  /* dataOnPio2 = (uint16_t*)(GPIO_GPIO2_BASE  */
  /* 				   | GPIO_IO_P4  */
  /* 				   | GPIO_IO_P5  */
  /* 				   | GPIO_IO_P8  */
  /* 				   | GPIO_IO_P10); */

  /* strobeOnPio3 = (uint16_t*)(GPIO_GPIO3_BASE  */
  /* 				     | GPIO_IO_P1); */
  /* clockOnPio3  = (uint16_t*)(GPIO_GPIO3_BASE */
  /* 				     | GPIO_IO_P2) */;

  gpioSetDir(RB_HB0, gpioDirection_Output);
  gpioSetDir(RB_HB2, gpioDirection_Output);
  gpioSetDir(RB_HB4, gpioDirection_Output);
  gpioSetDir(RB_HB5, gpioDirection_Output);
  gpioSetDir(RB_SPI_SS0, gpioDirection_Output);
  gpioSetDir(RB_SPI_SS1, gpioDirection_Output);
  gpioSetDir(RB_SPI_SS2, gpioDirection_Output);
  gpioSetDir(RB_SPI_SS5, gpioDirection_Output);
  gpioSetDir(RB_SPI_SS3, gpioDirection_Output);
  gpioSetDir(RB_SPI_SS4, gpioDirection_Output);
  
  // PIO1_3
  RB_HB0_IO &= ~IOCON_SWDIO_PIO1_3_FUNC_MASK ;
  RB_HB0_IO |= IOCON_SWDIO_PIO1_3_FUNC_GPIO;

  // PIO1_1
  RB_HB2_IO &= ~IOCON_JTAG_TDO_PIO1_1_FUNC_MASK ;
  RB_HB2_IO |= IOCON_JTAG_TDO_PIO1_1_FUNC_GPIO;

  // PIO1_4
  RB_HB4_IO &= ~IOCON_PIO1_4_FUNC_MASK ;
  RB_HB4_IO |= IOCON_PIO1_4_FUNC_GPIO;

  // PIO1_2
  RB_HB5_IO &= ~IOCON_JTAG_nTRST_PIO1_2_FUNC_MASK ;
  RB_HB5_IO |= IOCON_JTAG_nTRST_PIO1_2_FUNC_GPIO;
  
}

// Function to write a column to the BLINKENmini
void blinkenminiSetCol(bool column[8])
{
  //uint16_t tmpPixel;

  // Set the clock signal to low
  gpioSetValue(RB_SPI_SS3,0);

  // Set the strobe signal to low
  gpioSetValue(RB_SPI_SS4,0);

  // Write column data to the output port 1
  //tmpPixel = (column[0]<<3) | (column[1]<<1) | (column[2]<<4) | (column[3]<<2);
  //  *dataOnPio1 = tmpPixel;
  gpioSetValue(RB_HB0,column[0]);
  gpioSetValue(RB_HB2,column[1]);
  gpioSetValue(RB_HB4,column[2]);
  gpioSetValue(RB_HB5,column[3]);

  // Write column data to the output port 2
  //tmpPixel = (column[4]<<5) | (column[5]<<4) | (column[6]<<8) | (column[7]<<10);
  //*dataOnPio2 = tmpPixel;
  gpioSetValue(RB_SPI_SS0,column[4]);
  gpioSetValue(RB_SPI_SS1,column[5]);
  gpioSetValue(RB_SPI_SS2,column[6]);
  gpioSetValue(RB_SPI_SS5,column[7]);

  // Rising edge on the clock signal
  gpioSetValue(RB_SPI_SS3,1);
}

// Macro to finish a write to the BLINKENmini
// Set the strobe Signal to high
// we set it back to low in the SetCol function to achieve long pulses 
// (low frequency, better transmission on long lines)
void blinkenminiDisplay()
{
  gpioSetValue(RB_SPI_SS4,1);
}


